@@ -44,6 +44,16 @@ QT_BEGIN_NAMESPACE
     Indicates that a new channel got attached to the client.
 */
 
+/*!
+    \property QGrpcClientBase::channel
+    \since 6.7
+
+    This property holds the channel attached to this client. The channel is used
+    as the transport layer for gRPC operations.
+
+    \sa attachChannel()
+*/
+
 namespace {
 template <typename Operation>
 inline constexpr bool IsStream = false;
@@ -190,6 +200,12 @@ QGrpcClientBase::~QGrpcClientBase() = default;
 bool QGrpcClientBase::attachChannel(std::shared_ptr<QAbstractGrpcChannel> channel)
 {
     Q_D(QGrpcClientBase);
+
+    if (channel == d->channel) {
+        qGrpcWarning("Refusing to attach channel. The same channel is already assigned.");
+        return false;
+    }
+
     // channel is not a QObject so we compare against the threadId set on construction.
     if (channel->d_func()->threadId != QThread::currentThreadId()) {
         qGrpcWarning("QtGrpc doesn't allow attaching the channel from a different thread");
