@@ -97,8 +97,7 @@ QGrpcClientStream::~QGrpcClientStream() = default;
 */
 void QGrpcClientStream::writeMessage(const QProtobufMessage &message)
 {
-    QByteArray data = QGrpcOperation::context().serializer()->serialize(&message);
-    emit QGrpcOperation::context().writeMessageRequested(data);
+    QGrpcOperation::writeMessage(message);
 }
 
 /*!
@@ -110,7 +109,7 @@ void QGrpcClientStream::writeMessage(const QProtobufMessage &message)
 */
 void QGrpcClientStream::writesDone()
 {
-    emit QGrpcOperation::context().writesDoneRequested();
+    QGrpcOperation::writesDone();
 }
 
 bool QGrpcClientStream::event(QEvent *event)
@@ -165,8 +164,7 @@ QGrpcBidiStream::~QGrpcBidiStream() = default;
 */
 void QGrpcBidiStream::writeMessage(const QProtobufMessage &message)
 {
-    QByteArray data = QGrpcOperation::context().serializer()->serialize(&message);
-    emit QGrpcOperation::context().writeMessageRequested(data);
+    QGrpcOperation::writeMessage(message);
 }
 
 /*!
@@ -175,7 +173,7 @@ void QGrpcBidiStream::writeMessage(const QProtobufMessage &message)
 */
 void QGrpcBidiStream::writesDone()
 {
-    emit QGrpcOperation::context().writesDoneRequested();
+    QGrpcOperation::writesDone();
 }
 
 bool QGrpcBidiStream::event(QEvent *event)
